/* Application start
* 
* App initialize
*
*/
var myApp = angular.module('application', ['ionic', 'ionic-cache-src', 'ion-datetime-picker', 'angulartics', 'ionic.service.core', 'ngCordova', 'application.controllers', 'ngIOS9UIWebViewPatch', 'ionic-datepicker', 'application.validation', 'pascalprecht.translate'])
/* Application init */
.run(function($ionicPlatform, $rootScope, $state, Helper) {
	$rootScope.lang = {id:2, iso_code:"en"};//Set default language

	$ionicPlatform.ready(function() {
		if (window.cordova) {
			if(window.cordova.plugins.Keyboard) {
				cordova.plugins.Keyboard.hideKeyboardAccessoryBar(false);
				cordova.plugins.Keyboard.disableScroll(true);
			}

			var notificationOpenedCallback = function(jsonData) {
		      	Helper.log(jsonData.message);
		      	Helper.showAlertMessage(Helper.translate('generic_NOTIFICATION'), jsonData.message);
		    };

    		$rootScope.isDevice = true;

		    // OneSignal init
			window.plugins.OneSignal
			    .startInit(AppSettings.oneSignalAppId, {googleProjectNumber: AppSettings.oneSignalGoogleProjectNumber})
			    .handleNotificationOpened(notificationOpenedCallback)
			    .endInit();

	    	window.plugins.OneSignal.getIds(function(ids) {
			  	Helper.log('getIds: ' + JSON.stringify(ids));
			});

	    	// Pinterest init
	    	PinterestPlugin.init(AppSettings.pinterestApiKey);
		}
	});
})
.filter('reverse', function() {
  	return function(items) {
  		if(items)
    		return items.slice().reverse();
  	};
})
.directive('placeholderImg', function() {
    //default place holder
    return {
        restrict: 'A',
        link: function(scope, element, attr) {
            var img = window.scriptRoot + 'img/transparent.png';
            if (attr.placeholderImg) {
                switch (attr.placeholderImg) {
                    case 'attribute':
                        img = './img/transparent.png';
                        break;
                } 
            }
            //If ngSrc is empty
            if (!attr.ngSrc)
                element[0].src = img;
            //If there is an error (404)
            element.on('error', function() {
                element[0].src = img;
            });
        }
    };
})
/*  Default loading message */
.constant('$ionicLoadingConfig', {
	template: 'Loading...',
	noBackdrop: true
})
.config(['$provide', function($provide) {
    $provide.decorator('$locale', ['$delegate', function($delegate) {
        $delegate.NUMBER_FORMATS.DECIMAL_SEP = ',';
        return $delegate;
    }]);
	moment.locale('it');
}])
.config(function($stateProvider, $urlRouterProvider, $ionicConfigProvider, $httpProvider, $windowProvider) {
	$ionicConfigProvider.backButton.text('').previousTitleText(false);
	$ionicConfigProvider.views.swipeBackEnabled(false);

	var $window = $windowProvider.$get();
	var tablet = $window.innerWidth > 800 && $window.innerHeight > 600 ? true : false;

	$stateProvider
	.state('app', {
		url: '/app',
		abstract: true,
		templateUrl: function() {
			if (tablet) {
				return 'templates/tablet/menu.html'	
			} else {
				return 'templates/menu.html'
			}
		},
		controller: 'AppCtrl'
	})
	.state('app.home', {
		url: '/home',
		views: {
			'menuContent': {
				templateUrl: function() {
					if (tablet) {
						return 'templates/tablet/home.html'	
					} else {
						return 'templates/home.html'
					}
				},
				controller: 'HomeCtrl'
			}
		},
		cache: false
	})
	.state('app.signup', {
		url: '/signup',
		views: {
			'menuContent': {
				templateUrl: 'templates/signup.html',
				controller: 'SignupCtrl'
			}
		},
		cache: false
	})
	.state('app.personal', {
		url: '/personal',
		views: {
			'menuContent': {
				templateUrl: 'templates/personal.html',
				controller: 'PersonalCtrl'
			}
		},
		cache: false
	})
	.state('app.mod-password', {
		url: '/personal/mod-password',
		views: {
			'menuContent': {
				templateUrl: function() {
					if (tablet) {
						return 'templates/tablet/mod-password.html'	
					} else {
						return 'templates/mod-password.html'
					}
				},
				controller: 'ProfileCtrl'
			}
		},
		cache: false
	})
	.state('app.profile', {
		url: '/personal/profile',
		views: {
			'menuContent': {
				templateUrl: 'templates/profile.html',
				controller: 'ProfileCtrl'
			}
		},
		cache: false
	})
	.state('app.orders', {
		url: '/personal/orders',
		views: {
			'menuContent': {
				templateUrl: 'templates/orders.html',
				controller: 'OrdersCtrl'
			}
		},
		cache: false
	})
	.state('app.orderdetail', {
		url: '/personal/order/:id_order',
		views: {
			'menuContent': {
				templateUrl: function() {
					if (tablet) {
						return 'templates/tablet/orderdetail.html'	
					} else {
						return 'templates/orderdetail.html'
					}
				},
				controller: 'OrderDetailCtrl'
			}
		},
		cache: false
	})
	.state('app.addresses', {
		url: '/personal/addresses',
		views: {
			'menuContent': {
				templateUrl: 'templates/addresses.html',
				controller: 'AddressesCtrl'
			}
		},
		cache: false
	})
	.state('app.contacts', {
		url: '/personal/contacts',
		views: {
			'menuContent': {
				templateUrl: 'templates/contacts.html',
				controller: 'ContactsCtrl'
			}
		},
		cache: false
	})
	.state('app.addressdetail', {
		url: "/personal/address/{id_address}",
		views: {
			'menuContent': {
				templateUrl: function() {
					if (tablet) {
						return 'templates/tablet/addressdetail.html'	
					} else {
						return 'templates/addressdetail.html'
					}
				},
				controller: 'AddressDetailCtrl'
			}
		},
		cache: false
	})
	.state('app.cms', {
		url: '/cms/{id_cms}',
		views: {
			'menuContent': {
				templateUrl: 'templates/cms.html',
				controller: 'CmsCtrl'
			}
		},
		cache: false
	})
	.state('app.cms-parent', {
		url: '/parent/{id_parent}',
		views: {
			'menuContent': {
				templateUrl: 'templates/cms-parent.html',
				controller: 'CmsParentCtrl'
			}
		},
		cache: false
	})
	.state('app.login', {
		url: '/login',
		views: {
			'menuContent': {
				templateUrl: 'templates/login.html',
				controller: 'LoginCtrl'
			}
		},
		cache: false
	})
	.state('app.resetpw', {
		url: '/resetpw',
		views: {
			'menuContent': {
				templateUrl: 'templates/resetpw.html',
				controller: 'LoginCtrl'
			}
		},
		cache: false
	})
	.state('app.cart', {
		url: '/cart',
		views: {
			'menuContent': {
				templateUrl: function() {
					if (tablet) {
						return 'templates/tablet/cart.html'	
					} else {
						return 'templates/cart.html'
					}
				},
				controller: 'CartCtrl'
			}
		},
		cache: false
	})
	.state('app.ordercheckout', {
		url: '/ordercheckout',
		views: {
			'menuContent': {
				templateUrl: function() {
					if (tablet) {
						return 'templates/tablet/ordercheckout.html'	
					} else {
						return 'templates/ordercheckout.html'
					}
				},
				controller: 'OrderCheckoutCtrl'
			}
		},
		cache: false
	})
	.state('app.news', {
		url: '/news',
		views: {
			'menuContent': {
				templateUrl: 'templates/news.html',
				controller: 'NewsCtrl'
			}
		},
		cache: false
	})
	.state('app.newsdetail', {
		url: '/news/:id_news',
		views: {
			'menuContent': {
				templateUrl: function() {
					if (tablet) {
						return 'templates/tablet/newsdetail.html'	
					} else {
						return 'templates/newsdetail.html'
					}
				},
				controller: 'NewsDetailCtrl'
			}
		},
		cache: false
	})
	.state('app.stores', {
		url: '/stores',
		views: {
			'menuContent': {
				templateUrl: function() {
					if (tablet) {
						return 'templates/tablet/stores.html'	
					} else {
						return 'templates/stores.html'
					}
				},
				controller: 'StoresCtrl'
			}
		},
		cache: false
	})
	.state('app.storedetail', {
		url: '/store/:id_store',
		views: {
			'menuContent': {
				templateUrl: function() {
					if (tablet) {
						return 'templates/tablet/storedetail.html'	
					} else {
						return 'templates/storedetail.html'
					}
				},
				controller: 'StoreDetailCtrl'
			}
		},
		cache: false
	})
    .state('app.categories', {
        url: '/categories',
        views: {
            'menuContent': {
                templateUrl: 'templates/categories.html',
                controller: 'CategoriesCtrl'
            }
        },
		cache: false
    })
    .state('app.subcategories', {
        url: '/subcategories/:id_category',
        views: {
            'menuContent': {
                templateUrl: 'templates/subcategories.html',
                controller: 'SubcategoriesCtrl'
            }
        },
		cache: false
    })
    .state('app.products', {
        url: '/products/:id_category',
        views: {
            'menuContent': {
                templateUrl: function() {
					if (tablet) {
						return 'templates/tablet/products.html'	
					} else {
						return 'templates/products.html'
					}
				},
                controller: 'ProductsCtrl'
            }
        },
		cache: false
    })
    .state('app.productdetail', {
        url: '/products/:id_category/:id_product',
        views: {
            'menuContent': {
            	templateUrl: function() {
					if (tablet) {
						return 'templates/tablet/productdetail.html'	
					} else {
						return 'templates/productdetail.html'
					}
				},
                controller: 'ProductDetailCtrl'
            }
        },
		cache: false
    });

	$httpProvider.defaults.transformRequest.push(function (data, headersGetter) {
        var headers = headersGetter();
        angular.extend(headers, 'Authorization: basic ' + AppSettings.apiKey);

        return data;
    });

	$urlRouterProvider.otherwise("/app/home");
});