angular.module('application.controllers')
.controller('AppCtrl', function($rootScope, $scope, $state, $ionicConfig, $ionicPlatform, $cordovaDevice, $ionicPopup, $cordovaGoogleAnalytics, $cordovaFacebook, $translate, $ionicModal, $localStorage, $ionicHistory, $interval, $timeout, $window, $cordovaNetwork, Customers, Helper, Carts, RestService, Categories, Languages, News) {
	document.addEventListener("deviceready", function() {
		$rootScope.device = $cordovaDevice.getDevice();
		Helper.getDeviceType();
		$scope.device = $rootScope.device;
		$rootScope.isDevice = true;

		if(AppSettings.debug)
			$cordovaGoogleAnalytics.debugMode();
		$cordovaGoogleAnalytics.startTrackerWithId(AppSettings.analyticsAppId);
	}, false);

	$rootScope.$on('$cordovaNetwork:online', function(event, networkState) {
		$rootScope.device.online = true;

		if($rootScope.offlineModal) {
			$rootScope.offlineModal.hide();
			$rootScope.offlineModal = null;
		}
		//Flag for getting server base data in $localStorage
		if(!$rootScope.okBaseData !== true) {
			$scope.init();
		}
	});

	$rootScope.$on('$cordovaNetwork:offline', function(event, networkState) {
		Helper.loaderHide();

		$rootScope.device.online = false;

		if(!$rootScope.offlineModal) {
			$ionicModal.fromTemplateUrl('templates/offline.html', {
				animation: 'slide-in-up',
				scope: $scope,
				controller: 'AppCtrl'
			}).then(function(modal) {
				$rootScope.offlineModal = modal;
				modal.show();
			});
		}
	});

	$rootScope.$watch('categories', function() {
  		$scope.categories = $rootScope.categories;
  	});

	$scope.$on('$ionicView.enter', function(e) {
		$scope.isLogged = Customers.isLogged();
		var stateName = $ionicHistory.currentView().stateName;
		var stateParams = $ionicHistory.currentView().stateParams;
		if($rootScope.historyStack)
			$rootScope.historyStack.push({stateName:stateName, stateParams:stateParams});

		if($rootScope.isDevice && stateName)
			$cordovaGoogleAnalytics.trackView(stateName);

		$scope.siteUrl = AppSettings.siteUrl;
		$scope.imagesBaseUrl = AppSettings.imagesBaseUrl;
		$scope.imagesBaseUrlCategories = AppSettings.imagesBaseUrlCategories;
		$scope.imagesBaseUrlProducts = AppSettings.imagesBaseUrlProducts;

		if($rootScope.historyStack && $rootScope.historyStack.length > 1 && stateName != 'app.productdetail' && stateName != 'app.login' && $rootScope.historyStack[$rootScope.historyStack.length - 2].stateName != 'app.login') {
			$rootScope.filter_options = {};
			$rootScope.option_selected = [];
		}

		$scope.isCartView = stateName == 'app.cart';
		Helper.log(stateName);

		if($scope.isCartView) {
			$scope.isModCartActive = false;
			$scope.cartQtClass = '';

			$scope.toggleModCart = function() {
				$scope.isModCartActive = !$scope.isModCartActive;
				$scope.cartQtClass = $scope.isModCartActive ? 'item-select' : '';
			}
		}
		$scope.news = $localStorage.getObject('news');
	});

	$scope.go = function(path, data) {
		$state.go(path, data);
	}

	$scope.initMaintenanceCtrl = function() {
		// Init modal
		$ionicModal.fromTemplateUrl('templates/maintenance.html', {
			animation: 'slide-in-up',
			scope: $scope,
			controller: 'AppCtrl'
		}).then(function(modal) {
			$rootScope.maintenanceModal = modal;
		});
		
		// Control NOW and every 60 seconds if the app is in maintenance mode
		$scope.maintenance();
		$interval($scope.maintenance, 60000);
	}

	$scope.maintenance = function() {
		RestService.request('custom', {action: "maintenanceMode", version: AppSettings.apiVersion}, 'GET', null, null)
		.then(
			function(data) {
				$rootScope.maintenance = data.maintenance;

				if(data.maintenance == true || !data.version) {
					var id_cms = data.maintenance == true ? AppSettings.id_cms_maintenance : AppSettings.id_cms_new_version;

					RestService.request('custom', {action: "cms", id_cms: id_cms}, 'GET')
					.then(
						function(data) {
							$scope.maintenance = data;
							$rootScope.maintenanceModal.show();
						}
					)
					.finally(
						function() {
							Helper.loaderHide();
						}
					)
				}
				else {
					$rootScope.maintenanceModal.hide();
				}
			},
			function(error) {
				$rootScope.maintenanceModal.hide();
			}
		);
	}

	$scope.setLanguage = function() {
		Helper.getLanguage(function(lang) {
			Helper.log("setLanguage LANG: " + JSON.stringify(lang));
			if(lang) {
				$rootScope.lang = lang;
				Helper.log("setLanguage from device: " + $rootScope.lang.iso_code);
			}
			else {
				Helper.log("ERROR in setLanguage!!!");
				Helper.log(JSON.stringify(lang));
			}
			$scope.initAfterLang();
		});
	}

	$scope.initAfterLang = function() {
		News.list()
		.then(
			function(data) {
				if(data.length) {
					$scope.news = _.map(data, function(item) {
						if(item.image)
							item.image = item.image.replace("/list/", "/");
						return item;
					});
				}
			}
		)
		.finally(
			function() {
			   	//$ionicSlideBoxDelegate.update();
			}
		);
		
		var normalResources = [
			'currencies',
			'product_options',
			'product_option_values',
			'taxes',
			'tax_rules',
			'order_states'
		]

		var resources = [
			{
				id: 'categories',
				params: {display:"full", "filter[active]":1, "filter[is_mobile_visible]":1},
				cacheKey: 'categories',
				psKey: 'categories'
			},
			{
				id: 'currencies',
				params: {display:"full"},
				cacheKey: 'currencies',
				psKey: 'currencies'
			},
			{
				id: 'countries',
				params: {display:"full", "filter[active]":1},
				cacheKey: 'countries',
				psKey: 'countries'
			},
			{
				id: 'carriers',
				params: {display:"full", "filter[deleted]":0, "filter[active]":1},
				cacheKey: 'carriers',
				psKey: 'carriers'
			},
			{
				id: 'states',
				params: {display:"full", "filter[active]":1, "filter[id_country]":10},
				cacheKey: 'states',
				psKey: 'states'
			}
		];

		//Map all normal display:full resources
		_.map(normalResources, function(item) {
			resources.push({
				id: item,
				params: {display:"full"},
				cacheKey: item,
				psKey: item
			})
		})

		$scope.initMaintenanceCtrl();

		// TABLET
		$scope.isProductsOpen = false;
		$scope.openCategory = null;

		Helper.loaderShow();
		$window.async.each(
			resources,
			function(resource, callback) {
				RestService.request(resource.id, resource.params, 'GET', resource.cacheKey, resource.psKey)
				.then(
					function(data) {
            			callback(null);
					},
					function(error) {
						callback(error);
					}
				);
			},
			function(err) {
				$rootScope.okBaseData = true;

				$scope.categories = _.sortBy(_.where(Categories.stored(), {id_parent:AppSettings.rootCategoryId}), function(item) { return parseInt(item.position); });
				$scope.allCategories = _.sortBy(Categories.stored(), function(item) { return parseInt(item.position); });
				$scope.subcategories = {};

				//Get Tax
				$rootScope.tax = Helper.getTax();
				
				_.each($scope.categories, function(category) {
					$scope.subcategories[category.name] = _.where($scope.allCategories, {id_parent: category.id.toString()});
				});
				Helper.loaderHide();

				$state.go($state.current, {}, {reload: true});
			}
		);
		Carts.last(true);
	}

	//Load all resources in $localStorage
	$scope.init = function() {
		//Set Base Urls
		AppSettings.siteUrl = AppSettings.baseUrl;
		AppSettings.apiUrl = AppSettings.baseUrl + '/api/';
		AppSettings.imagesBaseUrl = AppSettings.baseUrl + '/p/';
		AppSettings.imagesBaseUrlStores = AppSettings.baseUrl + '/img/';
		AppSettings.imagesBaseUrlCategories = AppSettings.baseUrl + '/img/c/';
		AppSettings.imagesBaseUrlProducts = AppSettings.baseUrl + '/img/p/';
		AppSettings.imagesBaseUrlAttributes = AppSettings.baseUrl + '/img/co/';

		$rootScope.modal = {};
		$rootScope.historyStack = [];//History of user navigation

		$ionicConfig.navBar.alignTitle('center');

		$rootScope.device = {online:true};
		Helper.getDeviceType();

		$rootScope.$watch('lang', function() {
			$translate.use($rootScope.lang.iso_code);
		});
		$rootScope.$watch('productsInCart', function() {
			$scope.productsInCart = $rootScope.productsInCart;
		});
		$rootScope.$watch('totalProductsQt', function() {
			$scope.totalProductsQt = $rootScope.totalProductsQt;
		});

		Languages.list()
		.then(
			function(data) {
				
			},
			function(error) {
				
			}
		)
		.finally(
			function() {
				//Set device language if active
				$scope.setLanguage();
			}
		);
	}

	$scope.logout = function() {
		var confirmPopup = $ionicPopup.confirm({
			title: Helper.translate('login_LOGOUT'),
			template: Helper.translate('login_LOGOUT_CONFIRM'),
			okText: Helper.translate('generic_YES'),
			cancelText: Helper.translate('generic_NO')
		});

		confirmPopup.then(function(res) {
			if(res) {
				$window.async.waterfall([
					function(callback) {
						if($rootScope.isDevice) {
							$cordovaFacebook.getLoginStatus()
							.then(
								function(data) {
									callback(null, data);
								},
								function(error) {
									callback(error);
								}
							)
						}
						else {
							callback(null, null);
						}
				    },
				    function(result, callback) {
						if($rootScope.isDevice) {
					    	if(result.status == 'connected') {
					    		$cordovaFacebook.logout()
					    		.then(
									function(data) {
										callback(null, data);
									},
									function(error) {
										callback(error);
									}
				    			)
					    	}
					    	else {
					    		callback(null, null);
					    	}
				    	}
				    	else {
				    		callback(null, null);
				    	}
			    	},
			    	function(error) {
			    		if(error) {

			    		}
			    		else {
							Customers.logout();

							Helper.toast('login_EXITED');

					  		if($state.current.name == 'app.home') {
					  			$state.go($state.current, {}, {reload: true});
					  		}
					  		else {
					            $ionicHistory.nextViewOptions({
					                disableBack: true
					            });
					  			$state.go('app.home');
					  		}
			    		}
			    	}
		    	]);
			}
		});
	}
	
	$scope.toggleProducts = function() {
		Helper.log("toggleProducts was:" + $scope.isProductsOpen);
		$scope.isProductsOpen = !$scope.isProductsOpen;
		Helper.log("toggleProducts NOW is:" + $scope.isProductsOpen);
	}

	$scope.isCategoryOpen = function(category) {
		return ($scope.openCategory == category);
	}

	$scope.toggleCategory = function(category) {
		Helper.log("toggleCategory: " + category);
		$scope.openCategory = category;
	}

	$scope.init();
})