angular.module('application.controllers')
.controller('CartCtrl', function(
	$rootScope, 
	$scope, 
	$state, 
	$window, 
	$localStorage, 
	Helper, 
	Carts, 
	Products, 
	Customers, 
	Addresses, 
	Combinations, 
	Orders, 
	RestService
) {
	$scope.doRefresh = function() {
		$scope.init();
	   	$scope.$broadcast('scroll.refreshComplete');
	};
	
	$scope.changeQuantity = function(id_product_attribute, del, product_quantity) {
		if(del) {
			product_quantity = 0;//From select value
		}

		Helper.loaderShow();
		//Change combination quantity in the cart
		RestService.request('custom', {
			action: 'change_quantity', 
			id_cart: $scope.cart.id,
			id_product_attribute: id_product_attribute,
			secure_key: Helper.secureKey(),
			quantity: product_quantity
		}, 'GET')
		.then(
			function(data) {
    			$scope.refreshCart(data, true);
			},
			function(error) {
				callback(error);
			}
		)
		.finally(
			function() {
				Helper.loaderHide();
			}
		);
	}

	$scope.refreshCart = function(data, verifyPromo) {
		$scope.cart = data;
		$scope.cart.products = _.map($scope.cart.products, function(item) {
			item.imageUrl = AppSettings.imagesBaseUrl + item.id_image.replace('-','/') + '-medium_default.jpg';
			return item;
		});
		Carts.setCart($scope.cart);

		if(verifyPromo)
			$scope.verifyPromoCode();
	}

	$scope.range = function(min, max) {
	    var r = [];
	    for (var i = min; i <= max; i ++) {
	        r.push(i);
	    }
	    return r;
	};

	$scope.verifyPromoCode = function(goNext) {
		$scope.validPromoCode = false;
		Helper.loaderShow();

		Carts.verifyPromoCode({
			code: $scope.data.promoCode,
			id_customer: Customers.getSession() ? parseInt(Customers.getSession().id) : null,
			id_cart: (Customers.getSession() && $scope.cart) ? $scope.cart.id : null,
			id_lang: $rootScope.lang.id
		})
		.then(
			function(data) {
				$scope.promo = null;
				$scope.validPromoCode = false;
				$scope.cart.total_discounts = 0;
				var voucherCode = '';

				if(data.id_cart_rule) {
					var tax = Helper.getTax();//IVA
					var tax_diff = parseFloat(data.minimum_amount_tax) == 1 ? 0 : tax;// Apply tax or not

					if(parseFloat($scope.cart.total_paid) >= parseFloat(data.minimum_amount) * (1 + tax_diff)) {
						var promo = data;
						$scope.validPromoCode = true;

						$scope.cart.total_promo = $scope.cart.total_paid;

						if(parseFloat(promo.reduction_percent) > 0)  {
							promo.total = parseFloat($scope.cart.total_paid) * (100 - parseInt(promo.reduction_percent)) / 100;
							promo.label = "- " + parseInt(promo.reduction_percent) + "%";
							$scope.cart.total_discounts = $scope.cart.total_paid * parseInt(promo.reduction_percent) / 100;
						}
						else if(parseFloat(promo.reduction_amount) > 0) {
							var reductionCurrency = _.where($localStorage.getObject('currencies'), {id: parseInt(promo.reduction_currency)})[0];
							promo.total = parseFloat($scope.cart.total_paid) - parseFloat(promo.reduction_amount);
							promo.label = "- " + reductionCurrency.sign + " " + promo.reduction_amount;
							$scope.cart.total_discounts = parseFloat(promo.reduction_amount);
						}
						voucherCode = $scope.data.promoCode;
						$scope.promo = promo;
					}
				}
				else {
					$scope.cart.total_promo = 0;
					$scope.cart.total_discounts_tax_excl = 0;
				}

				//Voucher Control
				Carts.setVoucherCode('');
				if($scope.data.promoCode == "" || !$scope.data.promoCode) {

				}
				else if(voucherCode) {
					Carts.setVoucherCode(voucherCode);
				}
				else {
					Helper.genericError('order_NOT_VALID_PROMO');
				}
			},
			function(error) {
				Helper.log(error);
			}
		)
		.finally(
			function() {
				Orders.setPromo($scope.promo);
				Helper.loaderHide();

				RestService.request('custom', {
					action: 'cart_quantity', 
					id_customer: Customers.getSession() ? Customers.getSession().id : 0,
					secure_key: Helper.secureKey()
				}, 'GET')
				.then(
					function(data) {
		    			$scope.refreshCart(data);
					},
					function(error) {
						Helper.log(error);
					}
				)
				.finally(
					function() {
						Helper.loaderHide();
					}
				);

				if(goNext && ($scope.validPromoCode || !$scope.data.promoCode)) {
					if(Customers.getSession()) {
						$state.go('app.ordercheckout');
					}
					else {
						$state.go('app.login');
					}
				}
			}
		)
	}

	$scope.next = function() {
		$scope.verifyPromoCode(true);
	}

	$scope.init = function() {
		$scope.data = {};
		$scope.data.promoCode = Carts.getVoucherCode();
		$scope.formData = {};

		Helper.loaderShow();
		//Get data for each combination present in the cart
		RestService.request('custom', {
			action: 'cart_quantity', 
			id_customer: Customers.getSession() ? Customers.getSession().id : 0,
			secure_key: Helper.secureKey()
		}, 'GET')
		.then(
			function(data) {
    			$scope.refreshCart(data, true);
			},
			function(error) {
				Helper.log(error);
			}
		)
		.finally(
			function() {
				Helper.loaderHide();
			}
		);
	}

	$scope.$on('$ionicView.enter', function(e) {
		$scope.init();
	});
})