angular.module('application.controllers')
.controller('OrderCheckoutCtrl', function(
	$rootScope, 
	$scope, 
	$state, 
	$window, 
	$ionicPopup, 
	$ionicHistory, 
	$localStorage, 
	$cordovaGoogleAnalytics, 
	$cordovaInAppBrowser, 
	$ionicScrollDelegate,
	$ionicModal, 
	$cordovaCalendar,
	Helper, 
	Orders, 
	OrderDiscounts, 
	Carts, 
	Customers, 
	Products, 
	Categories, 
	Addresses, 
	Payments, 
	XPay, 
	RestService
) {	
	$scope.openModalPrivacy = function() {
		Helper.loaderShow();
		RestService.request('custom', {action: "cms", id_cms: 12}, 'GET')
		.then(
			function(data) {
				$scope.privacy = data;

				$ionicModal.fromTemplateUrl('privacy-modal.html', {
					scope: $scope,
					animation: 'slide-in-up'
				}).then(function(modal) {
					$scope.privacyModal = modal;
					$scope.privacyModal.show();
				});
			},
			function(error) {

			}
		)
		.finally(
			function() {
				Helper.loaderHide();
			}
		)
	}

	$scope.closeModalPrivacy = function() {
		$scope.privacyModal.hide();
	}

	$scope.getPostalCodes = function() {
		$scope.changePickupStore(true);
	}

	$scope.changePickupStore = function(empty) {
		if(empty) {
			$scope.data.store = null;
		}

		$scope.$broadcast('scroll.resize');

		if($scope.data.postal_code && ('' + $scope.data.postal_code).length >= 5) {
			Helper.loaderShow();
			RestService.request('custom', {
				action:'pickup_stores', 
				zip: $scope.data.postal_code,
				id_carrier: $scope.data.id_carrier,
			}, 'GET')
			.then(
				function(data) {
	    			$scope.pickup_stores = data;

	    			if($scope.data.store) {
	    				$scope.setPickupStores();
					}
				},
				function(error) {
	    			console.log(error);
				}
			)
			.finally(
				function() {
					Helper.loaderHide();

					$scope.$broadcast('scroll.resize');
				}
			);
		} else {
			$scope.data.store = null;
			$scope.pickup_stores = [];
		}
	}

	$scope.setPickupStores = function() {	
		var store = _.where($scope.pickup_stores, {'id_store':$scope.data.store})[0];

		$scope.data.date_at = store.calendar.selected_date;

		var enabledDates = [];
		var firstDay = null;
		var lastDay = null;

		_.map(store.calendar.dates, function(month) {
			_.map(month.weeks, function(week) {
				_.map(_.where(week, {disabled: false}), function(d) {
					if(moment(d.date, "YYYY-MM-DD").weekday() == 6) {
						enabledDates.push({ 'outside': { 'initial': d.date, 'final': d.date } });
					}
					if(!firstDay) {
						firstDay = moment(d.date, "YYYY-MM-DD").subtract(1, 'days').format("YYYY-MM-DD");
						enabledDates.push({ 'after': firstDay });
					}
					else {
						lastDay = moment(d.date, "YYYY-MM-DD").add(1, 'days').format("YYYY-MM-DD");
					}
				});
			});
		});
		enabledDates.push({ 'before': moment(lastDay, "YYYY-MM-DD").subtract(2, 'days').format("YYYY-MM-DD") });
		enabledDates.push({ 'inclusive': true });

		$scope.validDates = enabledDates;
	}

	$scope.getNewOrderParams = function(confirmOrder) {
		$scope.cart = Carts.getLastCart();
		if($scope.data.id_carrier) {
			$scope.cart.id_carrier = $scope.data.id_carrier;
		}
		var cart = $scope.cart;
		var payment = $scope.data ? Payments.get($scope.data.payment, true) : null;
		var currency = _.where($localStorage.getObject('currencies'), {id:parseInt(cart.id_currency)})[0];
		var discount = cart.total_discounts ? parseFloat(cart.total_discounts) : 0;

		Helper.log($scope.data.id_reference);

		if($scope.data.id_carrier == AppSettings.id_pickup_store) {
			if(!$scope.pickup_stores && $scope.data.postal_code) {
				Helper.loaderShow();
				RestService.request('custom', {
					action:'pickup_stores', 
					zip: $scope.data.postal_code,
					id_carrier: $scope.data.id_carrier,
				}, 'GET')
				.then(
					function(data) {
	        			$scope.pickup_stores = data;
					},
					function(error) {
	        			console.log(error);
					}
				)
				.finally(
					function() {
						Helper.loaderHide();
					}
				);
			}
			else if($scope.data.store && $scope.data.date_at) {
				Helper.loaderShow();
				var params = {
					action:'pickup_set', 
					date_pickup: moment($scope.data.date_at).format('YYYY-MM-DD HH:mm:ss'),
					id_store: $scope.data.store,
					id_cart: $scope.cart.id,
				};

				RestService.request('custom', params, 'GET')
				.then(
					function(data) {
						$scope.cart = data;
						$scope.data.address = data.id_address_delivery;
					},
					function(error) {
	        			console.log(error);
					}
				)
				.finally(
					function() {
						Helper.loaderHide();
					}
				);
			}
		} else {
			RestService.request('custom', {
				action: 'pickup_unset', 
				id_cart: $scope.cart.id, 
			}, 'GET')
			.then(
				function(data) {
					console.log(data);
				},
				function(error) {
					console.log(error);
				}
			)
			.finally(
				function() {
					Helper.loaderHide();
				}
			);
		}

		if($scope.data.address && $scope.data.id_carrier) {
			Helper.loaderShow();
			$window.async.series({
				address: function(callback) {
					RestService.request('custom', {
						action: 'address', 
						id: $scope.data.address, 
					}, 'GET')
					.then(
						function(data) {
							$scope.data.id_country = data.id_country;
		        			callback(null, data);
						},
						function(error) {
							callback(error);
						}
					);
				},
				payments: function(callback) {
					RestService.request('custom', {
						action: 'payments', 
						id_country: $scope.data.id_country, 
						id_carrier: $scope.data.id_carrier
					}, 'GET', 'payments')
					.then(
						function(data) {
		        			callback(null, data);
						},
						function(error) {
							callback(error);
						}
					);
				},
				cart: function(callback) {
					RestService.request('custom', {
						action:'cart_quantity', 
						id_cart:cart.id, 
						id_address: $scope.data.address, 
						id_carrier: $scope.data.id_carrier,
						payment: payment ? payment.module : null
					}, 'GET')
					.then(
						function(data) {
		        			callback(null, data);
						},
						function(error) {
							callback(error);
						}
					);
				}
			},
			function(error, result) {
				if(error) {
					Helper.genericError(err);
				}
				else if(result) {
					//update cart
					Carts.mod(result.cart)
					.then(
						function(data) {
							if(result.cart.id) {
								$scope.cart = result.cart;
								Carts.setCart($scope.cart);

								$scope.payments = result.payments;
								$scope.calculateParams($scope.cart, payment);

								Helper.log("PARAMS:");
								Helper.log($scope.params);

								if(confirmOrder == true && !result.cart.changed) {
									$scope.confirmOrder($scope.params);
								}
								else {
									Helper.loaderHide();
									if(result.cart.changed) {
										$ionicHistory.goBack();
										Helper.genericError('cart_WARNING_QUANTITY');
									}
									$ionicScrollDelegate.resize();
									$scope.$broadcast('scroll.resize');
								}
							}
						},
						function(error) {
							Helper.log("Errore durante la modifica del carrello");
							Helper.log(error);
							Helper.loaderHide();
						}
					)
				}
			});
		}
		else {
			$scope.calculateParams(cart, payment);

			if(confirmOrder) {
				$scope.confirmOrder($scope.params);
			}
		}
	}

	$scope.confirmOrder = function(params) {
		Orders.add(params)
		.then(
			function(data) {
				$scope.nextCart();

				if(!data.order) {
					Helper.log("NOT ORDER DATA!");
					Helper.loaderHide();
					Helper.genericError('order_USED_CART');
					$state.go('app.cart');
				}
				else {
					$scope.closeConfirmedOrder(data);
				}
			},
			function(error) {
				Helper.loaderHide();
				Helper.genericError(error);
			}
		);
	}

	$scope.calculateParams = function(cart, payment) {
		var params = {
			conversion_rate: 1,
			id_address_delivery: cart.id_address_delivery,
			id_address_invoice: cart.id_address_invoice,
			id_carrier: cart.id_carrier,
			id_cart: cart.id,
			id_currency: cart.id_currency,
			id_customer: cart.id_customer,
			id_lang: cart.id_lang,
			module: payment ? payment.module : null,
			payment: payment ? payment.payment : null,

			total_products: cart.total_products,
			total_products_wt: cart.total_products_wt,

			total_shipping: cart.total_shipping,
			total_shipping_tax_excl: cart.total_shipping_tax_excl,
			total_shipping_tax_incl: cart.total_shipping_tax_incl,

			total_discounts: cart.total_discounts,
			total_discounts_tax_excl: cart.total_discounts_tax_excl,
			total_discounts_tax_incl: cart.total_discounts_tax_incl,

			total_paid: cart.total_paid,
			total_paid_tax_incl: cart.total_paid_tax_incl,
			total_paid_tax_excl: cart.total_paid_tax_excl,
			total_paid_real: 0,
			valid: (payment && payment.module == 'codfee') ? 0 : 1,
			device: JSON.stringify($rootScope.device)
		};

		if(cart.id_carrier) {
			var ref = _.where($localStorage.getObject('carriers'), { id: parseInt(cart.id_carrier) });
			if(ref.length > 0) {
				$scope.data.id_reference = ref[0].id_reference;
			}
		}

		$scope.params = params;

		return params;
	}

	$scope.storesList = function() {
		Addresses.stores()
		.then(
			function(data) {
				$scope.stores = _.map(data, function(item) {
					item.country = _.where($localStorage.getObject('countries'), {id:parseInt(item.id_country)})[0];
					item.state = _.where($localStorage.getObject('states'), {id:parseInt(item.id_state)})[0];
					return item;
				});
			},
			function(error) {
				Helper.genericError(error);
			}
		)
		.finally(
			function() {
				Helper.loaderHide();
			}
		);
	}

	$scope.nextCart = function() {
		$scope.cart = null;
		Carts.setCart(null);
		Carts.setVoucherCode('');//Empty last valid voucher code
	  	$rootScope.totalProductsQt = 0;
	  	$rootScope.virtualCart = {};
	  	$scope.totalProductsQt = 0;
		Helper.log("Empty cart");
		Helper.loaderHide();
	}

	$scope.closeAnalyzeOrder = function(data) {
		Helper.log("Order analyze");
		Helper.log(data);

		var currency = _.where($localStorage.getObject('currencies'), {id:parseInt(data.order.id_currency)})[0];
		if($rootScope.isDevice) {
			$cordovaGoogleAnalytics.addTransaction(data.order.id, 'Mangano', Helper.lessTax(data.order.total_paid), '0', Helper.calculateTax(data.order.total_paid), currency.iso_code);

			angular.forEach(data.order.associations.order_rows, function(item, key) {
				Products.get(item.product_id)
				.then(
					function(products) {									
						var product = products.products[0];

						var category = _.where(Categories.stored(), {id: product.associations.categories[0].id})[0];
						if(!category)
							category = _.where(Categories.stored(), {id: parseInt(product.associations.categories[0].id)})[0];

						var parentCategory = _.where(Categories.stored(), {id: parseInt(category.id_parent)})[0];

					  	Helper.log("ANALYTICS TRANSACTION");
					  	Helper.log(data.order.id + " ," + item.product_name + " ," + item.product_reference + " ," + parentCategory.name + " - " + category.name + " ," + item.unit_price_tax_incl + " ," + '1' + " ," + currency.iso_code);
						
						$cordovaGoogleAnalytics.addTransactionItem(
						    data.order.id, item.product_name, item.product_reference, parentCategory.name + " - " + category.name, item.unit_price_tax_incl, '1', currency.iso_code
					  	);
					},
					function(error) {
						Helper.log("ERROR ANALITICS TRANSACTION");
					}
				);
		  	});
	  	}
	
		$ionicHistory.nextViewOptions({
		    disableBack: true
	  	});
		$state.go('app.orderdetail', {id_order:data.order.id});

  		Helper.toast('order_COMPLETED_SUCCESSFULLY');
	}

	$scope.closeConfirmedOrder = function(data, id_cart) {
		//Analytics track Transaction
		if(data) {
			$scope.closeAnalyzeOrder(data);
		}
		else {
			Helper.log("X-Pay/PayPal Cart");
			//Get last order data for X-Pay and PayPal
			Orders.getByIdCart(id_cart)
			.then(
				function(data) {
					if(data && data.length) {
						var orderData = data[0];
						if($rootScope.device) {
							//Set device in order
							orderData.device = JSON.stringify($rootScope.device);
							if($scope.cart && $scope.cart.id_address_delivery)
								orderData.id_address_delivery = orderData.id_address_invoice = $scope.cart.id_address_delivery;
							orderData.id_customer = Customers.getSession().id;

							Helper.log("SET DEVICE IN ORDER DATA")
							Helper.log("OrderData: " + JSON.stringify(orderData));

							Orders.mod(orderData)
							.then(
								function(modData) {
									$scope.nextCart();
									$scope.closeAnalyzeOrder(modData);
								},
								function(error) {
									Helper.log("ERROR UPDATE ORDER BY CART!");
									Helper.loaderHide();
								}
							)
						}
						else {
							$scope.closeAnalyzeOrder(orderData);
							$scope.nextCart();
						}
					}
					else {
						Helper.log('ERROR UPDATE ORDER BY CART ID: ' + id_cart);
						$scope.nextCart();
					}
				},
				function(error) {
					Helper.log("ERROR UPDATE ORDER!");
					Helper.loaderHide();
				}
			);
		}
	}

	$scope.next = function() {
		var payment = Payments.get($scope.data.payment, true);
		if(!payment) {
			$scope.data.payment = null;
		}
		if(!Addresses.stored()) {
			$scope.goAddAddress();
		}
		else if($scope.data.payment && $scope.data.id_carrier && $scope.data.address) {
			if($scope.data.privacy == true) {
				var cart = $scope.cart;
				var discount = $scope.params.total_discounts;
				var total_shipping = $scope.params.total_shipping;

				var total_paid = $scope.params.total_paid;
				var currency = _.where($localStorage.getObject('currencies'), {id:parseInt(cart.id_currency)})[0];

				if(payment) {
					Helper.loaderShow();
					Helper.log("PAYMENT METHOD: " + payment.module);

					if(payment.module == 'paypal') {
						var url = AppSettings.siteUrl + "/modules/paypal/express_checkout/payment.php";
						url += '?express_checkout=payment_cart';
	                    url += '&current_shop_url=' + AppSettings.siteUrl + '/ordine-veloce?';
	                    url += '&bn=PRESTASHOP_EC';
	                    url += '&id_cart=' + cart.id;

	                    Helper.log("PAYPAL OPEN GET Url: " + url);
	                    Orders.setCheckoutData($scope.data);
	                    Carts.setCart(cart);
						$cordovaInAppBrowser.open(url, '_blank', {location: 'no', clearcache: 'yes', toolbar: 'yes'});

						// Pages listener
						$rootScope.$on('$cordovaInAppBrowser:loadstart', function(e, event) {
							Helper.log("URL loadstart: " + event.url);
							if(Helper.getUrlParam(event.url, 'paypal_ec_canceled') == 1) {
								Helper.genericError('payments_PAYPAL_ERROR');

								$cordovaInAppBrowser.close();
							}
							else if(Helper.isSiteUrl(event.url)) {
								if(Helper.getUrlParam(event.url, 'id_module') && Helper.getUrlParam(event.url, 'id_cart') && Helper.getUrlParam(event.url, 'id_order')) {
									Helper.log("Order completed: " + Helper.getUrlParam(event.url, 'id_order'));

									$cordovaInAppBrowser.close();
									$scope.closeConfirmedOrder(null, Helper.getUrlParam(event.url, 'id_cart'));
								}
							}
					  	});

						$rootScope.$on('$cordovaInAppBrowser:exit', function(e, event) {
							Helper.loaderHide();
					  	});
					}
					else if(payment.module == 'pagxpcsprs') {
						XPay.init(cart.id, total_paid, currency.iso_code)
						.then(
							function(url) {
								$cordovaInAppBrowser.open(url.data, '_blank', {location: 'no', clearcache: 'yes',  toolbar: 'yes'});
							},
							function(error) {
								Helper.genericError('payments_XPAY_ERROR');
							}
						)

						// Pages listener
						$rootScope.$on('$cordovaInAppBrowser:loadstart', function(e, event) {
							Helper.log("URL loadstart: " + event.url);
							var esito = Helper.getUrlParam(event.url, 'esito');
							
							if(esito && $cordovaInAppBrowser)
								$cordovaInAppBrowser.close();
							
							switch(esito) {
								case 'ANNULLO':
									break;
								case 'ERRORE':
									Helper.genericError('payments_XPAY_ERROR');
									break;
								case 'OK':
								case 'TRANSACTION_OK':
									$scope.closeConfirmedOrder(null, cart.id);
							}
					  	});

						$rootScope.$on('$cordovaInAppBrowser:exit', function(e, event) {
							Helper.loaderHide();
					  	});
					}
					else if(payment.module == 'codfee') {
						$scope.getNewOrderParams(true);
					}
					else {
						Helper.loaderHide();
						Helper.genericError('order_PCA_ORDER');
					}
				}
			}
			else {
				Helper.genericError('order_NO_ACCEPTED_PRIVACY');
			}
		}
		else {
			Helper.genericError('order_PCA_ORDER');
		}
	}

	$scope.goAddAddress = function() {
		Orders.setCheckoutData($scope.data);
		$state.go('app.addressdetail');
	}

	$scope.init = function() {
		$scope.data = Orders.getCheckoutData();
		if($scope.data.id_carrier) {
			var ref = _.where($localStorage.getObject('carriers'), { id: parseInt($scope.data.id_carrier) });
			if(ref.length > 0) {
				$scope.data.id_reference = ref[0].id_reference;
			}
		}
		$scope.cart = Carts.getLastCart();
		$scope.promo = Orders.getPromo();

		$scope.id_default_carrier = AppSettings.id_default_carrier;
		$scope.id_pickup_store = AppSettings.id_pickup_store;

		Addresses.list()
		.then(
			function(data) {
				$scope.addresses = _.map(data, function(item) {
					item.country = _.where($localStorage.getObject('countries'), {id:parseInt(item.id_country)})[0];
					item.state = _.where($localStorage.getObject('states'), {id:parseInt(item.id_state)})[0];
					return item;
				});
			},
			function(error) {
				Helper.genericError(error);
			}
		);
		$scope.carriers = $localStorage.getObject('carriers');
		$scope.carriers = _.map($scope.carriers, function(item) {
			item.image = AppSettings.baseUrl + '/img/tmp/carrier_mini_' + item.id + '_1.jpg';
			return item;
		})
		
		Helper.loaderShow();
		RestService.request('custom', {action: "payments"}, 'GET')
		.then(
			function(data) {
				$scope.payments = data;
			},
			function(error) {
				$scope.payments = $localStorage.getObject('payments');
			}
		)
		.finally(
			function() {
				Helper.loaderHide();
			}
		)

		$scope.stores = $scope.storesList();
		$scope.getNewOrderParams();
	}

	$scope.$watch('data.date_at',function (oldValue, newValue) {
		$scope.getNewOrderParams();
	});

	$scope.$watch('data.store',function (oldValue, newValue) {
		$scope.getNewOrderParams();
	});

	$scope.$on('$ionicView.enter', function(e) {
		$scope.init();
		$scope.getPostalCodes();
	});

	$scope.init();
})