angular.module('application.controllers')
.controller('SignupCtrl', function($rootScope, $scope, $ionicPopup, $ionicHistory, $window, $state, Errors, Helper, RestService, Customers, Carts) {
	$scope.data = {
		active: 1,
		id_shop: AppSettings.id_shop,
		id_shop_group: AppSettings.id_shop_group,
		id_default_group: AppSettings.id_default_group,
		id_lang: $rootScope.lang.id,
		id_gender: 1
	};

	$scope.addUser = function(params) {
		Helper.loaderShow();
		Customers.add(params)
		.then(
			function(data) {
				Helper.log("User signed up");
				$scope.data = {
					email: params.email,
					passwd: params.passwd,
				}
				$scope.silentLogin();
			},
			function(error) {
				if(error.data.errors[0].code == 97 || parseInt(error.data.errors[0].code) == 46) {
					Helper.genericError('signup_ERROR_DUPLICATE');	
				}
				else {
					Helper.genericError(error);
				}
			}
		)
		.finally(
			function() {
				Helper.loaderHide();
			}
		);
	}

	$scope.signup = function() {
		var params = JSON.parse(JSON.stringify($scope.data));

		if(params.birthday) {
			params.birthday = moment(params.birthday, ['DD/MM/YYYY', 'YYYY/MM/DD']).format("YYYY-MM-DD");
		}
		if(!params.passwd) {
			Helper.genericError('login_ERROR_NO_PASSWORDS');
		}
		else if(params.passwd.length < 8) {
			Helper.genericError('login_ERROR_LENGTH_PASSWORDS');
		}
		else if(params.passwd != params.passwd_repeat) {
			Helper.genericError('login_ERROR_PASSWORDS');
		}
		else {
			if(params.newsletter == 1) {
				RestService.request('custom', {
					action: 'newsletter-add',
					firstname: params.firstname,
					lastname: params.lastname,
					email: params.email
				}, 'GET')
				.then(
					function(data) {
						Helper.log(data);
						$scope.addUser(params);
					}
				);
			}
			else {
				$scope.addUser(params);
			}
		}
	}

	$scope.silentLogin = function() {
		if(!$scope.data.email || !$scope.data.passwd) {
			Helper.genericError('login_ERROR_ACCESS_INSERT_DATA');
		}
		else {
			var params = JSON.parse(JSON.stringify($scope.data));

			Helper.loaderShow();
			Customers.login(params)
			.then(
				function(data) {
					if(data && data.active) {
						Customers.setSession(data);

						Helper.toast('login_WELCOME');

						Helper.goBack(2);
					}
					else {
						Helper.genericError('login_ERROR_ACCESS');
					}
				},
				function(error) {
					Helper.genericError('login_ERROR_ACCESS');
				}
			)
			.finally(
				function() {
					Helper.loaderHide();
				}
			);
		}
	}
})