angular.module('application.controllers')
.controller('StoresCtrl', function($rootScope, $scope, $timeout, $ionicPopup, $window, $cordovaGeolocation, Helper, Stores, RestService) {
	$scope.doRefresh = function() {
		$scope.init();
	   	$scope.$broadcast('scroll.refreshComplete');
	   	$scope.$apply();
	}

    $scope.switchView = function(viewType) {
    	$scope.viewType = viewType;
    	if(viewType == 'map') {
    		$scope.loadMap($scope.stores);
    	}
    }    

    $scope.setPoints = function(points, map) {
        var latlngbounds = new google.maps.LatLngBounds();
        var itemLatLng = null;
        var infowindow = null;
        var marker = null;

    	angular.forEach(points, function(item) {
			itemLatLng = new google.maps.LatLng(item.latitude, item.longitude);

	        marker = new google.maps.Marker({
	          	position: itemLatLng,
	          	map: map,
	          	icon: "img/map_marker.png",
	          	title: item.name
	        });
			infowindow = new google.maps.InfoWindow();

	        google.maps.event.addListener(marker, 'click', function() {
		        if(item.distance) {
		        	infowindow.setContent('<div>\
			        						<a href="#/app/store/' + item.id_store + '">\
			        							<b>' + item.name + '</b>\
			    							</a>\
			    							<div>' + Helper.translate('stores_FROM_DISTANCE', {distance:item.distance}) + '</div>\
	    								</div>');
		        }
		        else {
		        	infowindow.setContent('<div>\
				    						<a href="#/app/store/' + item.id_store + '">\
				    							<b>' + item.name + '</b>\
											</a>\
										</div>');
		        }
	          	infowindow.open(map, this);
	        });
    		latlngbounds.extend(itemLatLng);
    	});		
		map.setCenter(latlngbounds.getCenter());
		map.fitBounds(latlngbounds);
		
    	Helper.loaderHide();
    }

    $scope.loadMap = function(items) {
	  	$cordovaGeolocation.getCurrentPosition({
			enableHighAccuracy: false,
			timeout: 20000,
			maximumAge: 0
		})
	    .then(function (position) {
	      	var gpsPosition = position.coords;
	      	$rootScope.gpsPosition = gpsPosition;
			var positionLatLng = new google.maps.LatLng(gpsPosition.latitude, gpsPosition.longitude);

	        var mapOptions = {
	          	center: positionLatLng,
	          	zoom: 16,
	          	mapTypeId: google.maps.MapTypeId.ROADMAP,
	          	mapTypeControl: false,
	          	zoomControl: false
	        };
	        var map = new google.maps.Map(document.getElementById("map"), mapOptions);

	        if(gpsPosition.latitude && gpsPosition.longitude) {
	        	var d = null;
	        	for(var i = 0; i < items.length; i++) {
	        		d = Helper.distance(gpsPosition.latitude, gpsPosition.longitude, items[i].latitude, items[i].longitude);
    				distance = parseFloat(d) < 1000 ? d + " m" : parseFloat(d / 1000).toFixed(2) + " Km";
    				items[i].distance = distance;
	        	}

	    		$window.async.sortBy(items, function(x, callback) {
				    callback(null, parseFloat(x.distance));
				}, function(err,result){
				    $scope.stores = result;
				});
    		}
			$scope.setPoints(items, map);
	    }, function(error) {

	    });

	    $timeout(function() {
		    if(!$rootScope.gpsPosition) {
		    	//Helper.showAlertMessage(Helper.translate('stores_ENABLE_GPS_TITLE'), Helper.translate('stores_ENABLE_GPS_BODY'));
				
		        var mapOptions = {
		          	mapTypeId: google.maps.MapTypeId.ROADMAP,
		          	mapTypeControl: false,
		          	zoomControl: false
		        };
		        var map = new google.maps.Map(document.getElementById("map"), mapOptions);
				$scope.setPoints(items, map);
		    }
	    }, 1000);
	}
	$scope.setMarkerPosition = function(marker, position) {
	    marker.setPosition(
	        new google.maps.LatLng(
	            position.coords.latitude,
	            position.coords.longitude
            )
	    );
	}

    $scope.viewType = "list";//default

	$scope.init = function() {
		Helper.loaderShow();

		RestService.request('custom', {
			action:'stores'
		}, 'GET')
		.then(
			function(data) {
				$scope.stores = data;
				$scope.loadMap($scope.stores);
			},
			function(error) {
				Helper.genericError(error);
			}
		)
		.finally(
			function() {
				Helper.loaderHide();
			}
		);
	}

	$scope.init();
})