/*
 * Application utils
 * =================
 */

angular.module('application.utils', [])
.service('RestService', function($http, $rootScope, $localStorage, $q, $log, Helper) {
    return {
        generateXmlData: function(resource, params, wrapper) {
            var data = {prestashop:{}};
            var wrap = wrapper ? wrapper : resource;

            data.prestashop[wrap] = params;

            var x2js = new X2JS({
                arrayAccessFormPath: "prestashop.cart.cart_rows.cart_row"
            });

            return x2js.json2xml_str(data);
        },
        request: function(resource, params, method, cacheKey, psKey) {
            var deferred = $q.defer();
            var service = this;
            var url = AppSettings.apiUrl + resource;
            var data = {};

            if(method == 'POST' || method == 'PUT') {
                data = service.generateXmlData(resource, params);
                params = {};
            }
            else if(method == 'DELETE') {
                url += '/' + params.id;
                params = {};
            }
            params.ws_key = AppSettings.apiKey;
            params.output_format = 'JSON';
            params.language = $rootScope.lang.id;

            if(method == 'GET')
                params.hash_id = Math.random();// No cache call

            var req = {
                url: url,
                method: method,
                params: params,
                headers: {
                    "Content-Type": "text/plain, */*"
                },
                data: data,
                timeout: 30000
            };

            $http(req)
            .then(
                function(response) {
                    var data = psKey ? response.data[psKey] : response.data;
                    
                    if(cacheKey) {
                        $localStorage.setObject(cacheKey, data);
                    }
                    deferred.resolve(data);
                },
                function(error) {
                    deferred.reject(error);
                }
            );
            return deferred.promise;
        }
    }
})
.factory('$localStorage', function($window) {
    return {
        set: function(key, value) {
            $window.localStorage[key] = value;
        },
        get: function(key, defaultValue) {
            return $window.localStorage[key] || defaultValue;
        },
        setObject: function(key, value) {
            $window.localStorage[key] = JSON.stringify(value);
        },
        getObject: function(key) {
            try {
                return JSON.parse($window.localStorage[key] || null)
            }
            catch(e) {
                return null;
            }
        },
        delete: function(key) {
            if(key && $window.localStorage[key])
                $window.localStorage.removeItem(key);
        }
    }
});

/*
 * Application helpers
 * ===================
 */
angular.module('application.helpers', ['application.controllers'])
.factory('Helper', function(
    $window, 
    $sce, 
    $q, 
    $log, 
    $rootScope, 
    $state, 
    $filter, 
    $http, 
    $ionicPopup, 
    $ionicHistory, 
    $ionicLoading, 
    $localStorage, 
    $window, 
    $cordovaDevice, 
    Errors,
    $cordovaToast,
    $cordovaVibration
) {
    return {
        showAlertMessage: function(title, message) {
            $rootScope.popup = $ionicPopup.alert({
                title: title,
                template: message,
                okType: 'button-positive'
            });
        },
        showErrorAlertMessage: function(message) {
            this.showAlertMessage(this.translate('generic_ERROR'), message);
        },
        inArray: function(needle, haystack) {
            var length = haystack.length;
            for(var i = 0; i < length; i++) {
                if(haystack[i] == needle) return true;
            }
            return false;
        },
        toast: function(label) {
            if($rootScope.isDevice) {
                $cordovaToast.showShortBottom(this.translate(label));
                $cordovaVibration.vibrate(100);
            }
            else {
                this.log(this.translate(label));
            }
        },
        openMail: function(address) {
            if (ionic.Platform.isIOS()) {
                $window.location.href = $sce.trustAsResourceUrl('mailto:' + address);
            }
            if (ionic.Platform.isAndroid()) {
                $cordovaInAppBrowser.open('mailto:' + address, '_system');
            }
        },
        getImageUrl: function(relativeUrl) {
            return AppSettings.imagesBaseUrl + '/' + relativeUrl;
        },
        distance: function(lat1, lon1, lat2, lon2) {
            var p1 = new google.maps.LatLng(lat1, lon1);
            var p2 = new google.maps.LatLng(lat2, lon2);

            return (google.maps.geometry.spherical.computeDistanceBetween(p1, p2)).toFixed(2);
        },
        getUrlParam: function(url, variable) {
            var vars = url.split("?");
            if(vars.length > 1) {
                vars = vars[1].split("&");
                for(var i=0;i<vars.length;i++) {
                    var pair = vars[i].split("=");
                    if(pair[0] == variable) {
                        return (pair[1]);
                    }
                }
            }
            return(false);
        },
        secureKey: function() {
            $localStorage.get('secure_key');
            if(!$localStorage.get('secure_key')) {
                $localStorage.set('secure_key', Math.random().toString(36).substr(2));
            }
            return $localStorage.get('secure_key');
        },
        isSiteUrl: function(url) {
            var urlDomain = url.split("/");
            var siteDomain = AppSettings.siteUrl.split("/");

            return (urlDomain[2] == siteDomain[2]);
        },
        getDeviceType: function() {
            if(!$rootScope.device)
                $rootScope.device = {};
            if($window.innerWidth > 800 && $window.innerHeight > 600) {
                if($rootScope.isDevice) {
                    $window.screen.lockOrientation('landscape');
                }
                $rootScope.device.type =  'tablet';
            }
            else {
                if($rootScope.isDevice) {
                    $window.screen.lockOrientation('portrait');
                }
                $rootScope.device.type = 'mobile';
            }
            return $rootScope.device;
        },
        loaderShow: function() {
            $ionicLoading.show({
                template: this.translate('generic_LOADING')
            });
        },
        loaderHide: function() {
            $ionicLoading.hide();
        },
        log: function(message) {
            if(AppSettings.debug) {
                $log.log(message);
            }
        },
        genericError: function(errors) {
            if(typeof errors === 'string') {
                $rootScope.popup = $ionicPopup.alert({
                    title: this.translate('generic_ERROR'),
                    template: Errors[errors][$rootScope.lang.iso_code]
                });
            }
            else if(errors) {
                var message = Errors[errors.data.errors[0]['message']] ? Errors[errors.data.errors[0]['message']][$rootScope.lang.iso_code] : errors.data.errors[0].message;
                           
                $rootScope.popup = $ionicPopup.alert({
                    title: this.translate('generic_ERROR'),
                    template: message
                });
            }
        },
        getTax: function() {
            return parseFloat($localStorage.getObject('taxes')[0].rate) / 100;
        },
        applyTax: function(value) {
            return parseFloat(value) * (1 + this.getTax());
        },
        lessTax: function(value) {
            return parseFloat(value) / (1 + this.getTax());
        },
        calculateTax: function(value) {
            return value * $rootScope.tax;
        },
        round1: function(value) {
            return Math.round(parseFloat(value)*10)/10;
        },
        round2: function(value) {
            return Math.round(parseFloat(value)*100)/100;
        },
        getLanguage: function(callback) {
            var langCode = null;
            var lang = null;

            if(navigator.globalization) {
                navigator.globalization.getPreferredLanguage(
                    function(language) {
                        langCode = language.value.toString().substring(0, 2);
                        lang = _.where($localStorage.getObject('languages'), {iso_code:langCode})[0];
                        if(!lang) {
                            langCode = navigator.language.toString().substring(0, 2);
                            lang = _.where($localStorage.getObject('languages'), {iso_code:langCode})[0];
                            callback(lang);
                        }
                        callback(lang);
                    }
                );
            }
            else {
                langCode = navigator.language.toString().substring(0, 2);
                lang = _.where($localStorage.getObject('languages'), {iso_code:langCode})[0];
                callback(lang);
            }
        },
        translate: function(key, params) {
            var $translate = $filter('translate');
            return $translate(key, params); 
        },
        goBack: function(index) {
            var index = index ? index : 1;
            var backView = $rootScope.historyStack[$rootScope.historyStack.length-1 - index];

            $ionicHistory.nextViewOptions({
                disableBack: true
            });
            if(backView)
                $state.go(backView.stateName, backView.stateParams);
            else
                $state.go('app.home');
        }
    }
})

/*
 * Application models
 * ==================
 */
angular.module('application.models', ['application.utils', 'application.helpers'])
    .factory('Categories', function($localStorage, $filter, RestService) {
        return {
            get: function(id) {
                var categories = $localStorage.getObject('categories');

                return $filter('filter')(categories, {id: parseInt(id)}, true)[0];
            },
            list: function(id_parent) {
                var params = {display:"full", "is_mobile_visible":1, "filter[active]":1};
                if(id_parent)
                    params["filter[id_parent]"] = id_parent;

                var cacheKey = id_parent ? 'subcategories' : 'categories';
                
                return RestService.request('categories', params, 'GET', cacheKey, 'categories');
            },
            stored: function() {
                return _.sortBy($localStorage.getObject('categories'), function(item) { return parseInt(item.position); });
            }
        }
    })
    .factory('Customers', function($localStorage, $filter, RestService) {
        return {
            add: function(params) {
                params.associations = {groups:[{group:[{id:3}]}]};//Customers group
                return RestService.request('customers', params, 'POST');
            },
            mod: function(params) {
                var service = this;
                var data = service.getSession();
                if(!params.passwd) {
                    params.passwd = data.passwd;
                }
                params.associations = {groups:[{group:[{id:3}]}]};//Customers group

                return RestService.request('customers', params, 'PUT');
            },
            get: function(id) {
                var params = {display:"full", "filter[id]":id, "filter[active]":1};

                return RestService.request('customers', params, 'GET', null, 'customers');
            },
            list: function(id_parent) {
                var params = {display:"full"};
                
                return RestService.request('customers', params, 'GET', 'customers');
            },
            stored: function() {
                return $localStorage.getObject('customers');
            },
            login: function(params) {
                params.action = 'login';
                params.SubmitLogin = true;
                params.mobile = 1;
                return RestService.request('custom', params, 'GET');
            },
            loginFb: function(params) {
                params.action = 'loginFb';

                return RestService.request('custom', params, 'GET');
            },
            resetPassword: function(params) {
                params.action = 'resetPassword';

                return RestService.request('custom', params, 'GET');
            },
            getSession: function() {
                return $localStorage.getObject('session') ? $localStorage.getObject('session') : null;
            },
            setSession: function(data) {
                console.log("SESSION")
                console.log(data)
                return $localStorage.setObject('session', data);
            },
            isLogged: function() {
                var service = this;
                return Boolean(service.getSession());
            },
            logout: function() {
                $localStorage.delete('session');
            }
        }
    })
    .factory('Addresses', function($localStorage, $filter, RestService, Customers) {
        return {
            add: function(params) {
                params.id_customer = Customers.getSession().id;
                return RestService.request('addresses', params, 'POST');
            },
            mod: function(params) {
                return RestService.request('addresses', params, 'PUT');
            },
            del: function(id) {
                return RestService.request('custom', {action:'del_address', id_address:id}, 'GET');
            },
            get: function(id) {
                var addresses = $localStorage.getObject('addresses');

                return $filter('filter')(addresses, {id: parseInt(id)}, true)[0];
            },
            list: function() {
                var params = {display:"full", "filter[id_customer]":Customers.getSession().id, "filter[deleted]":0};
                
                return RestService.request('addresses', params, 'GET', 'addresses', 'addresses');
            },
            stores: function() {
                var params = {display:"full", "filter[id_customer]":0,"filter[alias]":"Pickup at Store", "filter[deleted]":0};

                return RestService.request('addresses', params, 'GET', null, 'addresses');
            },
            stored: function() {
                return $localStorage.getObject('addresses');
            }
        }
    })
    .factory('News', function($rootScope, $localStorage, $http, $filter) {
        return {
            get: function(id) {
                var news = $localStorage.getObject('news');
                
                return $filter('filter')(news, {id:id.toString}, true)[0];
            },
            list: function() {
                return $http({
                    method: 'GET',
                    url: AppSettings.siteUrl + "/modules/psblog/rss.php?id_lang=" + $rootScope.lang.id,
                    params: {
                        ws_key: AppSettings.apiKey
                    },
                    dataType: 'text/xml',
                    headers: {
                        'Accept': 'application/xml',
                        'Content-Type': 'application/xml'
                    }
                })
                .then(
                    function(response) {
                        var x2js = new X2JS();
                        var channel = x2js.xml_str2json(response.data).rss.channel;
                        var news = [];
                        if(channel.item) {
                            news = channel.item && channel.item.length > 1 ? channel.item : [channel.item];
                        }
                        $localStorage.setObject('news', news);

                        return news;
                    }
                );
            },
            stored: function() {
                return $localStorage.getObject('news');
            }
        }
    })
    .factory('Products', function($localStorage, RestService) {
        return {
            get: function(id) {
                var params = {display:"full", "filter[active]":1, "filter[id]":id, "filter[available_for_order]":1};
                
                return RestService.request('products', params, 'GET');
            },
            stockAvailable: function(id_product) {
                var params = {display:"full", "filter[id_product]":parseInt(id_product), "filter[id_product_attribute]":0};
                
                return RestService.request('stock_availables', params, 'GET', null, 'stock_availables');
            },
            list: function(id_category_default) {
                var params = {display:"full", "filter[active]":1, "filter[available_for_order]":1, "filter[id_category_default]":id_category_default};
                
                return RestService.request('products', params, 'GET', null, 'products');
            },
            stored: function() {
                return $localStorage.getObject('products');
            },
            videos: function(id) {
                var params = {
                    action: 'product_videos',
                    id: id
                };

                return RestService.request('custom', params, 'GET');
            },
            model3d: function(id) {
                var params = {
                    action: 'magic360',
                    id: id
                };

                return RestService.request('custom', params, 'GET');
            },
            verifyReduction: function(params) {
                params.action = 'specific_prices';
                return RestService.request('custom', params, 'GET');
            }
        }
    })
    .factory('Combinations', function($localStorage, RestService) {
        return {
            get: function(id) {
                var params = {display:"full", "filter[id]":id};
                
                return RestService.request('combinations', params, 'GET');
            },
            stockAvailable: function(id_product_attribute) {
                var params = {display:"full", "filter[id_product_attribute]":parseInt(id_product_attribute)};
                
                return RestService.request('stock_availables', params, 'GET', null, 'stock_availables');
            },
            list: function(id_product) {
                var params = {display:"full", "filter[id_product]":id_product};

                //Delete the last combination list
                if($localStorage.getObject('combinations')) {
                    $localStorage.delete('combinations');
                }
                
                return RestService.request('combinations', params, 'GET', 'combinations', 'combinations');
            },
            stored: function() {
                return $localStorage.getObject('combinations');
            }
        }
    })
    .service('Carts', function($localStorage, $rootScope, Helper, RestService, Customers) {
        var cartService = this;

        cartService.lastCart = {};
        cartService.voucherCode = '';
        
        cartService.mod = function(params) {
            return RestService.request('carts', params, 'PUT');
        }
        cartService.get = function(id) {
            var params = {display:"full", "filter[id]":id};
            
            return RestService.request('carts', params, 'GET');
        }
        cartService.setVoucherCode = function(voucherCode) {
            cartService.voucherCode = voucherCode;
        }
        cartService.getVoucherCode = function() {
            return cartService.voucherCode;
        }
        cartService.setCart = function(cart) {
            $rootScope.lastCart = cartService.lastCart = cart;

            if(cart && cart.products) {
                $rootScope.totalProductsQt = _.reduce(cart.products, function(memo, item) {
                    return memo + parseInt(item.quantity);
                }, 0);
            }
            else {
                $rootScope.totalProductsQt = 0;
            }
        }
        cartService.last = function(set) {
            if(Customers.getSession()) {
                return cartService.list()
                .then(
                    function(data) {
                        if(data && data.carts) {
                            var cart = data.carts[data.carts.length-1];

                            if(cart.associations != undefined && cart.associations.cart_rows != undefined) {
                                cart.associations.cart_rows = {cart_row: cart.associations.cart_rows};
                            }
                            else if(!cart) {
                                cart = {associations: {cart_rows: {cart_row: []}}};
                            }
                            else if(!cart.associations) {
                                cart.associations = {cart_rows: {cart_row: []}};
                            }
                            else if(!cart.associations.cart_rows) {
                                cart.associations.cart_rows = {cart_row: []};
                            }
                            if(set)
                                cartService.setCart(cart);

                            return cart;
                        }
                    },
                    function(error) {
                        Helper.log(error);
                    }
                )
            }
            else if(!$rootScope.virtualCart) {
                $rootScope.virtualCart = {associations: {cart_rows: {cart_row: []}}};
            }
        }
        cartService.verifyPromoCode = function(params) {
            params.action = 'cart_rules';
            
            params.secure_key = Helper.secureKey();

            return RestService.request('custom', params, 'GET');
        }
        cartService.getLastCart = function() {
            return cartService.lastCart;
        }
        cartService.list = function() {
            if(Customers.getSession()) {
                var params = {display:"full", "filter[id_customer]":Customers.getSession().id};
                
                return RestService.request('carts', params, 'GET', 'carts');
            }
        }
        cartService.stored = function() {
            return $localStorage.getObject('carts');
        }
    })
    .service('Languages', function($localStorage, RestService) {
        var languagesService = this;
        
        languagesService.list = function() {
            var params = {display:"full"};
            
            return RestService.request('languages', params, 'GET', 'languages', 'languages');
        }
        languagesService.stored = function() {
            return $localStorage.getObject('languages');
        }
    })
    .service('Carriers', function($localStorage, RestService) {
        var carrierService = this;
        
        carrierService.get = function(id) {
            var params = {display:"full", "filter[id]":id};
            
            return RestService.request('carriers', params, 'GET');
        }
        carrierService.list = function() {
            var params = {display:"full", "filter[deleted]":0};
            
            return RestService.request('carriers', params, 'GET', 'carriers', 'carriers');
        }
        carrierService.stored = function() {
            return $localStorage.getObject('carriers');
        }
    })
    .service('Payments', function($localStorage, RestService) {
        var paymentService = this;
        
        paymentService.get = function(id, stored) {
            if(stored) {
                return _.where(paymentService.stored(), {module:id})[0];
            }
            else {
                var params = {display:"full", "filter[id]":id};
                
                return RestService.request('payments', params, 'GET');
            }
        }
        paymentService.list = function() {
            var params = {display:"full"};
            
            return RestService.request('payments', params, 'GET', 'payments', 'payments');
        }
        paymentService.stored = function() {
            return $localStorage.getObject('payments');
        }
    })
    .service('XPay', function($http, Helper) {
        var xpayService = this;

        xpayService.init = function(id_cart, total_paid, currency) {
            var url = AppSettings.siteUrl + "/index.php?fc=module&module=pagxpcsprs&controller=payment";
            url += '&id_cart=' + id_cart;
            url += '&importo=' + total_paid;
            url += '&divisa=' + currency;
            url += '&redirect=false';
            var req = {
                url: url,
                method: 'GET',
                headers: {
                    "Content-Type": "text/plain, */*"
                }
            };

            return $http(req)
                .then(function(response) {
                    return response;
                });
        }
    })
    .service('Social', function($cordovaSocialSharing, $cordovaInstagram, $cordovaInAppBrowser, $window, $rootScope, Helper) {
        var socialService = this;

        socialService.getBase64Image = function(id) {console.log(id)
            var img = document.getElementById(id);
            var canvas = document.createElement("canvas");
            canvas.width = img.width;
            canvas.height = img.height;
            var ctx = canvas.getContext("2d");
            ctx.drawImage(img, 0, 0);
            var dataURL = canvas.toDataURL();
            return dataURL;
        }

        socialService.share = function(social, type, siteUrl, obj) {
            var postUrl = siteUrl;
            if(type == 'product')
                postUrl += '/index.php?id_product=' + obj.id + '&controller=product&id_lang=' + $rootScope.lang.id;
            else if(type == 'news')
                postUrl += '/index.php?fc=module&module=psblog&controller=posts&id_lang=' + $rootScope.lang.id + '&post=' + obj.id;

            var htmlBody = '<a href="' + postUrl + '">'+obj.title+'</a><br><br><img src="' + obj.image + '">' + obj.description;
            var postTitle = 'MANGANO' + " - " + obj.title;
            var postImageUrl = obj.image;

            var socialApps = {
                email: {
                    name:'E-Mail',
                    method:'shareViaEmail'
                },
                facebook: {
                    name:'Facebook',
                    method:'shareViaFacebook'
                },
                gplus: {
                    name:'Google Plus',
                    method:'shareViaGooglePlus'
                },
                twitter: {
                    name:'Twitter',
                    method:'shareViaTwitter'
                }, 
                pinterest: {
                    name:'Pinterest',
                    method:'shareViaPinterest'
                },
                instagram: {
                    name:'Instagram',
                    method:'shareViaInstagram'
                }
            };

            switch(social) {
                case 'email':
                    try {
                        $cordovaSocialSharing.shareViaEmail(htmlBody, postTitle, postImageUrl)
                        .then(
                            function(result) {
                                Helper.log(result);
                                //Helper.showAlertMessage(socialApps[social].name, Helper.translate('social_POST_SHARED_VIA') + ' ' + socialApps[social].name);
                            },
                            function(err) {
                                Helper.log(err);
                                Helper.showErrorAlertMessage(Helper.translate('social_ERROR_NOT_INSTALLED_APP') + ' ' + socialApps[social].name);
                            }
                        );
                    }
                    catch(e) {
                        Helper.showErrorAlertMessage(Helper.translate('social_ERROR_NOT_INSTALLED_APP') + ' ' + socialApps[social].name);
                    }
                    break;

                case 'facebook':
                    try {
                        $cordovaSocialSharing.shareViaFacebook(postTitle, postImageUrl, postUrl)
                        .then(
                            function(result) {
                                Helper.log(result);
                                //Helper.showAlertMessage(socialApps[social].name, Helper.translate('social_POST_SHARED_VIA') + ' ' + socialApps[social].name);
                            }, 
                            function(err) {
                                Helper.log(err);
                                Helper.showErrorAlertMessage(Helper.translate('social_ERROR_NOT_INSTALLED_APP') + ' ' + socialApps[social].name);
                            }
                        );
                    }
                    catch(e) {
                        Helper.showErrorAlertMessage(Helper.translate('social_ERROR_NOT_INSTALLED_APP') + ' ' + socialApps[social].name);
                    }
                    break;

                case 'twitter':
                    try {
                        $cordovaSocialSharing.shareViaTwitter(postTitle + " - " + postUrl, postImageUrl, postUrl)
                        .then(
                            function(result) {
                                //Helper.showAlertMessage(socialApps[social].name, Helper.translate('social_POST_SHARED_VIA') + ' ' + socialApps[social].name);
                            },
                            function(err) {
                                Helper.log(err);
                                Helper.showErrorAlertMessage(Helper.translate('social_ERROR_NOT_INSTALLED_APP') + ' ' + socialApps[social].name);
                            }
                        );
                    }
                    catch(e) {
                        Helper.showErrorAlertMessage(Helper.translate('social_ERROR_NOT_INSTALLED_APP') + ' ' + socialApps[social].name);
                    }
                    break;

                case 'gplus':
                    try {
                        $window.plugins.googleplus.isAvailable(
                            function(available) {
                                if (available) {
                                    var url = 'https://plus.google.com/share?url=' + postUrl;
                                    $cordovaInAppBrowser.open(url, '_blank', {location: 'yes', clearcache: 'yes', closebuttoncaption: Helper.translate('generic_CLOSE')});
                                }
                                else {
                                    Helper.genericError('social_NOT_INSTALLED_GPLUS');
                                }
                            }
                        );
                    }
                    catch(e) {
                        Helper.genericError('social_NOT_INSTALLED_GPLUS');
                    } 
                    break;

                case 'pinterest':
                    //try {
                        PinterestPlugin.pin(postUrl, postImageUrl, postTitle, 
                            function(result) {
                                Helper.log(result);
                            },
                            function(err) {
                                Helper.log(postUrl + "\n" + postImageUrl + "\n" + postTitle);
                                Helper.log(err);
                                Helper.showErrorAlertMessage(Helper.translate('social_ERROR_POST_SHARE_VIA') + ' ' + socialApps[social].name);
                            }
                        );
                    /*}
                    catch(e) {
                        Helper.genericError('social_NOT_INSTALLED_PINTEREST');
                    }*/
                    break;

                case 'instagram':
                    try {
                        var myBase64Image = socialService.getBase64Image("image_" + type + "_" + obj.id);

                        $cordovaInstagram.share({image:myBase64Image}, postTitle)
                        .then(
                            function(result) {
                                Helper.log(result);
                                //Helper.showAlertMessage(socialApps[social].name, Helper.translate('social_POST_SHARED_VIA') + ' ' + socialApps[social].name);
                            }, 
                            function(err) {
                                Helper.log(err);
                                Helper.genericError('social_NOT_INSTALLED_INSTAGRAM');
                            }
                        );
                    }
                    catch(e) {
                        Helper.genericError('social_NOT_INSTALLED_INSTAGRAM');
                    }
                    break;
            }
        }
    })
    .service('Orders', function($localStorage, RestService, Customers, Addresses, Carriers, Payments) {
        var orderService = this;

        orderService.payment = {};
        orderService.carrier = {};
        orderService.address = {};

        orderService.add = function(params) {
            return RestService.request('orders', params, 'POST');
        }
        orderService.mod = function(params) {
            return RestService.request('orders', params, 'PUT');
        }
        orderService.get = function(id) {
            var params = {display:"full", "filter[id]":parseInt(id)};
            
            return RestService.request('orders', params, 'GET', null, 'orders');
        }
        orderService.getByIdCart = function(id_cart) {
            var params = {display:"full", "filter[id_cart]":parseInt(id_cart)};
            
            return RestService.request('orders', params, 'GET', null, 'orders');
        }
        orderService.list = function() {
            var params = {display:"full", "filter[id_customer]":Customers.getSession().id};
            
            return RestService.request('orders', params, 'GET', 'orders', 'orders');
        }
        orderService.stored = function() {
            return $localStorage.getObject('orders');
        }
        orderService.setPayment = function(module) {
            orderService.payment = _.where(Payments.stored(), {module:module})[0];
        }
        orderService.setPromo = function(promo) {
            orderService.promo = promo;
        }
        orderService.setCarrier = function(id_carrier) {
            orderService.carrier = _.where(Carriers.stored(), {id:parseInt(id_carrier)})[0];
        }
        orderService.setAddress = function(id_address) {
            orderService.address = _.where(Addresses.stored(), {id:parseInt(id_address)})[0];
        }
        orderService.setCheckoutData = function(data) {
            orderService.checkoutData = data;
        }
        orderService.getPayment = function() {
            return orderService.payment;
        }
        orderService.getPromo = function() {
            return orderService.promo;
        }
        orderService.getCarrier = function() {
            return orderService.carrier;
        }
        orderService.getAddress = function() {
            return orderService.address;
        }
        orderService.getCheckoutData = function() {
            return orderService.checkoutData ? orderService.checkoutData : {};
        }
        orderService.getDeliveryPrice = function(params) {
            return RestService.request('custom', params, 'GET');
        }
    })
    .service('OrderDiscounts', function($localStorage, RestService) {
        var orderDiscountService = this;

        orderDiscountService.add = function(params) {
            return RestService.request('order_discounts', params, 'POST');
        }
        orderDiscountService.get = function(id) {
            var params = {display:"full", "filter[id]":parseInt(id)};
            
            return RestService.request('order_discounts', params, 'GET', null, 'order_discounts');
        }
        orderDiscountService.list = function() {
            var params = {display:"full", "filter[id_customer]":Customers.getSession().id};
            
            return RestService.request('order_discounts', params, 'GET', 'order_discounts', 'order_discounts');
        }
        orderDiscountService.stored = function() {
            return $localStorage.getObject('order_discounts');
        }
    })
    .factory('Stores', function($localStorage, RestService, Helper) {
        return {
            get: function(id) {
                var params = {display:"full", "filter[active]":1, "filter[id]":id};
                
                return RestService.request('stores', params, 'GET');
            },
            list: function() {
                var params = {display:"full", "filter[active]":1};
                
                return RestService.request('stores', params, 'GET');
            },
            times: function(str) {
                var times = str.split(";");
                var days = [
                    'stores_MONDAY',
                    'stores_TUESDAY',
                    'stores_WEDNESDAY',
                    'stores_THURSDAY',
                    'stores_FRIDAY',
                    'stores_SATURDAY',
                    'stores_SUNDAY'
                ];
                var result = [];
                for (var i = 0; i < times.length; i++) {
                    result.push({hour:times[i], day:Helper.translate(days[i])})
                }
                return result;
            },
            stored: function() {
                return $localStorage.getObject('stores');
            }
        }
    })